/*
 * HTApplet.java
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2005 Embrapa Informtica Agropecuria
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//package hypertree.demo;

//import hypertree.HyperTree;
//import hypertree.HTView;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.applet.*;
import java.awt.*;
import java.net.URL;
import  javax.swing.*;



/**
 * The Demo class implements a demo for HyperTree. It display an HyperTree
 * view of a demo tree.
 * The demo tree is build from a file tree, passed in parameter.
 * Demo could take an argument, the path from which 
 * start the representing of files.
 * If no arguments is given, the treemap start from the root.
 */

public class HTApplet extends Applet { // Appelet

   private static HTFileNode root      = null; // the root of the demo tree
   private static HyperTree  hypertree = null; // the hypertree builded
   private static HTView view; 
//   private Jpanel jpanel;
   TextField searchField;
   private Image imageH; 
   private Image imageB;
   private Image imageS;
   JFrame viewFrame;
   String htFile ;
   String htHintColor   ;
   String htCenterNode ;
   String htBackColor  ;
   String htLinkColor  ;
   String htSearchColor ;
   String htBorderType  ;
   String htFont       ;
   String htFontSize    ;
   String htHomeImage  ;
   String htHelpImage ;
   String htBackImage;
   String htHelpFile  ;
   String htUrlAdr  ;
   String pathRoot = null;

   private int undoControlAtual = 0;
   private int undoControlBase  = 0;

      
    public HTApplet() {		     	
    }

    public void init() {
  
        pathRoot = null;
	  htFile        = getParameter("htfile");	   
	  htHintColor   = getParameter("hthintcolor");	
	  htCenterNode  = getParameter("htcenternode");
	  htBackColor   = getParameter("htbackcolor");
	  htLinkColor   = getParameter("htlinkcolor");
        htSearchColor = getParameter("htsearchcolor");
	  htBorderType  = getParameter("htbordertype");
	  htFont        = getParameter("htfont");
        htFontSize    = getParameter("htfontsize");
        htHomeImage   = getParameter("hthomeimage");
	  htHelpImage   = getParameter("hthelpimage");
        htBackImage   = getParameter("htbackimage");
	  htHelpFile    = getParameter("hthelpfile");
	  htUrlAdr      = getParameter("hturladr");


	  if (htFile == null) {
	      System.out.println("Arquivo com a rvore Hiperblica no foi especificado");
		return;
	  }

//	try {
//	      if( Class.forName("netscape.security.PrivilegeManager") != null )
//    			netscape.security.PrivilegeManager.enablePrivilege("UniversalAwtEventQueueAccess");//
//	} catch (Exception e) {
//    		// privilege is not currently granted
//	}

  	  try {
		
            GZIPInputStream zip = new GZIPInputStream( (new URL( getCodeBase(), htFile )).openStream()); 
		ObjectInputStream pi = new ObjectInputStream(zip);
		root = (HTFileNode ) pi.readObject(); 
		zip.close();
 
 	  } catch (Exception e) {
        	System.out.println("Leitura de Objetos  :  " + e.toString());
		return;
        }
     
        if (root == null) {
            System.err.println("Erro : No foi possvel iniciar a rvore Hiperblica");
            return;
        }
 
        hypertree   = new HyperTree(root, this);       

        view = hypertree.getView();
	  view.setApplet(this);

	  if (htHintColor!= null)
		view.setHintColor(htHintColor);

	  if (htBackColor!= null)
		view.setBackColor(htBackColor);

	  if (htLinkColor!= null)
		view.setLinkColor(htLinkColor);

	  if (htSearchColor!= null)
		view.setSearchColor(htSearchColor);

	  if (htBorderType != null)
		view.setBorderType(htBorderType);

	  if (htCenterNode != null)
		view.setCenterNode(htCenterNode);

	  if (htHelpFile != null) {
		view.setHelpFile(htHelpFile);
	  }

	  if (htUrlAdr != null)
		view.setUrlAdr(htUrlAdr);

	  if (htFont != null)
		view.setFontNode(htFont);
	  else
		view.setFontNode("Serif");

	  if (htFontSize != null)
		view.setFontSizeNode(((new Integer(htFontSize)).intValue()) );
	  else
		view.setFontSizeNode(12);

	  if (htHomeImage != null) {
		imageH = getImage(getCodeBase(), htHomeImage);
		if (imageH != null) {
			view.setHomeImage(imageH);				
		}
	  }
	  else {
	  }

	  if (htBackImage != null) {
		imageB = getImage(getCodeBase(), htBackImage);
		if (imageB != null) {
			view.setBackImage(imageB);				
		}
	  }
	  else {
	  }

	  if (htHelpImage != null) {
		imageS = getImage(getCodeBase(), htHelpImage);
		if (imageS != null) {
			view.setHelpImage(imageS);				
		}
	  }
	  else {
	  }
	

	  // Swing - JPanel 

	  //Container content = getContentPane();
	    
	  //content.setLayout(new BorderLayout());

	  //content.setBackground(Color.gray); 

        //content.setSize(new Dimension(800, 600));

	  //content.add("Center", view); 
	 
	  //content.setVisible(true);

	  // AWT - PAnel
/*
	   searchField = new TextField(10);
         searchField.setBackground(Color.lightGray);
	   searchField.reshape(15, 20, 92, 24);

         view.setTextField(searchField); // **
	   searchField.addTextListener(new HTListenerText(view)); // this)); // **
  */
	  setLayout(new BorderLayout());
	  
	  setBackground(new Color (Color.decode(view.getBackColor()).getRGB())); 
       
        setSize(new Dimension(80, 60));

	  //add(searchField);

	  add("Center", view); 	  
	 
	  setVisible(true);
                   
	  viewFrame = new JFrame(root.getName());
        viewFrame.setContentPane(view);
        viewFrame.setSize(800,600);
	  //viewFrame.setRootPane(view);

        //viewFrame.pack();
        viewFrame.setVisible(true);

    }

    public final void reLoad(String center)
    {               
    
	  this.destroy();
	
        //hypertree   = new HyperTree(root, this);

	  hypertree.setNewHyperTree(root, this); 

	  view = null;

	  System.gc();
	  
        view = hypertree.getView();

	  view.setApplet(this);
	 
	  if (htHintColor!= null)
		view.setHintColor(htHintColor);

	  if (htBackColor!= null)
		view.setBackColor(htBackColor);

	  if (htLinkColor!= null)
		view.setLinkColor(htLinkColor);

	  if (htSearchColor!= null)
		view.setSearchColor(htSearchColor);

	  if (htBorderType != null)
		view.setBorderType(htBorderType);

	  
	  if (center != null)
		  view.setCenterNode(center);

	  if (htHelpFile != null) {
		view.setHelpFile(htHelpFile);
	  }

	  if (htUrlAdr != null)
		view.setUrlAdr(htUrlAdr);

	  if (htFont != null)
		view.setFontNode(htFont);
	  else
		view.setFontNode("Serif");

	  if (htFontSize != null)
		view.setFontSizeNode(((new Integer(htFontSize)).intValue()) );
	  else
		view.setFontSizeNode(12);

	  if (htHomeImage != null) {
		imageH = getImage(getCodeBase(), htHomeImage);
		if (imageH != null) {
			view.setHomeImage(imageH);				
		}
	  }
	  else {
	  }

	  if (htBackImage != null) {
		imageB = getImage(getCodeBase(), htBackImage);
		if (imageB != null) {
			view.setBackImage(imageB);				
		}
	  }
	  else {
	  }

	  if (htHelpImage != null) {
		imageS = getImage(getCodeBase(), htHelpImage);
		if (imageS != null) {
			view.setHelpImage(imageS);				
		}
	  }
	  else {
	  }
	/*
	   searchField = new TextField(10);
         searchField.setBackground(Color.lightGray);
	   searchField.reshape(15, 20, 92, 24);

         view.setTextField(searchField); // **
	   searchField.addTextListener(new HTListenerText(view)); // this)); // **
	*/
  
	  setLayout(new BorderLayout());
	  
	  setBackground(new Color (Color.decode(view.getBackColor()).getRGB())); 
       
        //setSize(new Dimension(800, 600));

	  //add(searchField);

	  add("Center", view); 	  
	 
	  setVisible(true);

        viewFrame.setContentPane(view);
        //viewFrame.pack();
        viewFrame.setVisible(true);

        //JRootPane.setContentPane(view);
        //viewFrame.pack();
        //JRootPane.setVisible(true);

	  percorre(root, 0);
    }

	public void percorre(HTFileNode node, int ind ) {

		String b = "";

		for (int i=0; i < ind; i++) 
			b = b + "    ";

		System.out.println(b + node.getID() + " " + node.getName() + " " + node.getColor()); 
		

		for (Enumeration e = node.children(); e.hasMoreElements(); ) {
            		HTFileNode childNode = (HTFileNode) e.nextElement();
            		if (!childNode.isLeaf()) {
				ind = ind + 1;
                		percorre(childNode, ind);
				ind = ind - 1;
            		}
			else {	
				ind = ind + 1;
				b = "";
				for (int i=0; i < ind; i++) 
					b = b + "    ";
				System.out.println(b+ " >> " + childNode.getID() + " " + childNode.getName() + " " + childNode.getHint()); 					
				ind = ind - 1;
			}
		}

	}


   
/*           
	
    public final void update(Graphics g)
    {
        
            super.paint(g);

    }
    public final void paint(Graphics g)
    {        
           
           super.paint(g);
    }

public void paint(Graphics g) {//conteudo 
	//g.setColor(Color.red); 
	//g.drawString("Eu sou uma applet",25,45); 
	//g.setColor(Color.green); 
	//g.drawString("que desenha um retangulo",45,75); 
	//g.setColor(Color.orange); 
	//g.drawRect(200,50,30,40); 
	view.paintComponent(g);

} 


public void start() {//conteudo 
	show();
	 requestFocus();
}


public void stop() {//conteudo 
	destroy();
}

public void paint(Graphics g) {//conteudo 
	g.setColor(Color.red); 
	g.drawString("Eu sou uma applet",25,45); 

	for (int i=0; i < 10000000; );

	view.paintComponent(g);
} 

*/

    public void undoRecupera() {
		undoControlAtual--;

		if (undoControlAtual < 0) {
			if (undoControlBase > 0)
				undoControlAtual = 9;
			else {
				undoControlAtual = 0;
				return;
			}
		}
		try {			
            	GZIPInputStream zip = new GZIPInputStream( (new URL( getCodeBase(), htFile+undoControlAtual )).openStream()); 
			ObjectInputStream pi = new ObjectInputStream(zip);
			root = (HTFileNode ) pi.readObject(); 
			zip.close();

		} catch (Exception e) {
            	System.out.println("\nGrava de Objetos  :"+
            	e.toString());
		}		
    }

    public void undoGrava() {
		
	try {			
		GZIPOutputStream zip  = new GZIPOutputStream(new FileOutputStream(htFile+undoControlAtual));
		ObjectOutputStream po = new ObjectOutputStream(zip); 				
		po.writeObject(root); 				
		zip.close();

	} catch (Exception e) {
            System.out.println("\nGrava de Objetos  :"+
            e.toString());
	}
	undoControlAtual++;	
	if (undoControlAtual > 9) {
		undoControlBase  = 1;
		undoControlAtual = 0;
	}	
    }

    public void grava() {
	try {			
		GZIPOutputStream zip  = new GZIPOutputStream(new FileOutputStream(htFile)); 
		ObjectOutputStream po = new ObjectOutputStream(zip); 				
		po.writeObject(root); 				
		zip.close();

	} catch (Exception e) {
            System.out.println("\nGrava de Objetos  :"+
            e.toString());
	}
	System.out.println("Hipertree Gravada.");		
    }


    public boolean hasUndo() {

		if ((undoControlAtual == 0) && (undoControlBase == 0)) {
			return false;
		}
		else
			return true;
    }


}

