/*
 * HTDoubleBuffer.java
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2005 Embrapa Informtica Agropecuria
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import java.awt.*;

public class HTDoubleBuffer extends Panel { // Jpanel ou Panel

	//	class variables
	private int bufferWidth;
	private int bufferHeight;
	private Image bufferImage;
	private Graphics bufferGraphics;

	public HTDoubleBuffer(){
		super();
	}

	public void update(Graphics g){
		if (g != null) paint(g);
	}

	public void paint(Graphics g){
		//	checks the buffersize with the current panelsize
		//	or initialises the image with the first paint
		if(	bufferWidth!=getSize().width ||
			bufferHeight!=getSize().height ||
			bufferImage==null ||
			bufferGraphics==null)
				resetBuffer();

		if(bufferGraphics!=null){
			//	this clears the offscreen image, not the onscreen one
			bufferGraphics.clearRect(0,0,bufferWidth,bufferHeight);

			//	calls the paintbuffer method with the offscreen graphics as a param
			paintBuffer(bufferGraphics);

			//	we finaly paint the offscreen image onto the onscreen image

			if (bufferImage != null)
				g.drawImage(bufferImage,0,0,this);
		}
	}

	private void resetBuffer(){
		// always keep track of the image size
		bufferWidth=getSize().width;
		bufferHeight=getSize().height;

		//	clean up the previous image
		if(bufferGraphics!=null){
			bufferGraphics.dispose();
			bufferGraphics=null;
		}
		if(bufferImage!=null){
			bufferImage.flush();
			bufferImage=null;
		}
		System.gc();

		//	create the new image with the size of the panel
		bufferImage=createImage(bufferWidth,bufferHeight);
		bufferGraphics=bufferImage.getGraphics();
	}

	public void paintBuffer(Graphics g){
		//	in classes extended from this one, add something to paint here!
		//	always remember, g is the offscreen graphics
	}

	
}